package zero1hd.rhythmbullet.audio.visualizer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Pixmap.Format;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Disposable;

import zero1hd.rhythmbullet.audio.MusicManager;

public class HorizontalVisualizer implements Disposable {
	private Pixmap pixmap;
	private Texture barTexture;
	private int barWidth;
	private int binsPerBar;
	private int spaceBetweenBars;
	private Sprite[] bars;
	private float[] barHeights;
	private int smoothRange;
	private float barHeightMultiplier;
	private float rotation;
	private Vector2 angleRot;
	private boolean flip;
	private Array<MirrorVisualizer> mirrors;
	private boolean reverse;
	private float maxAvgHeight;
	private float currentAvg;
	private int barCount;
	private float width, height, x, y;
	
	private Visualizer visualizer;
	
	public HorizontalVisualizer(Visualizer visualizer) {
		super();
		mirrors = new Array<>();
		pixmap = new Pixmap(2, 2, Format.RGBA8888);
		pixmap.setColor(Color.WHITE);
		pixmap.fill();
		
		barCount = 68;
		
		width = Gdx.graphics.getWidth();
		height = Gdx.graphics.getHeight()/2f;
		x = 0;
		y = 0;
		
		this.visualizer = visualizer;
		
		smoothRange = 2;
		angleRot = new Vector2(MathUtils.cosDeg(rotation), MathUtils.sinDeg(rotation));
		bars = new Sprite[barCount];
		barHeights = new float[barCount];
		barTexture = new Texture(pixmap);
		for (int i = 0; i < bars.length; i++) {
			bars[i] = new Sprite(barTexture);
		}
		
		updatePositionInfo();
		pixmap.dispose();
	}
	
	public void render(Batch batch, float parentAlpha) {
		if (visualizer.getMM() != null) {
			for (int i = 0; i < bars.length; i++) {
				bars[i].draw(batch);
				for (int j = 0; j < mirrors.size; j++) {
					mirrors.get(j).render(i, batch, parentAlpha, bars);
				}
			}
		}
		
	}
	
	public void update(float delta) {
		if (visualizer.getMM() != null) {
			//Averaging bins together
			for (int i = 0; i < barCount; i++) {
				float barHeight = 0;
				for (int j = 0; j < binsPerBar; j++) {
					barHeight += Math.abs(visualizer.getAudioPCMData()[j+i*binsPerBar +1]);
				}
				barHeight /= binsPerBar;
				barHeight *= barHeightMultiplier;
				barHeights[i] = barHeight;
			}
			currentAvg = 0;
			for (int i = 0; i < barCount; i++) {
				int avg = 0;
				//Averaging the bars
				for (int range = 0; range < smoothRange; range++) {
					if (i+range < barCount) {
						avg += barHeights[i+range];
					}
					if (i-range >= 0) {
						avg += barHeights[i-range];
					}
				}
				avg /= smoothRange*2;
				barHeights[i] = avg;
				currentAvg += barHeights[i];
				if (bars[i].getHeight() > barHeights[i]) {
					bars[i].setSize(barWidth, bars[i].getHeight() - (15f*delta*(bars[i].getHeight()-barHeights[i])));
				} else {
					bars[i].setSize(barWidth, bars[i].getHeight() + (20f*delta*(barHeights[i] - bars[i].getHeight())));
				}
			}
			currentAvg /= barHeights.length;
			if (currentAvg > maxAvgHeight) {
				maxAvgHeight = currentAvg;
			}
		}
	}
	
	public void setMM(MusicManager mm) {
		maxAvgHeight = 0;
		currentAvg = 0;
		float validBins = (4200/((mm.getSampleRate()/2)/((mm.getReadWindowSize()/2)+1)));
		Gdx.app.debug("Visualizer", "valid frequency bins " + validBins);
		binsPerBar = MathUtils.round((validBins/barCount));
		barHeights = new float[barCount];
		visualizer.setMM(mm);
	}
	
	public void dispose() {
		barTexture.dispose();
		visualizer.dispose();
	}
	
	public void updatePositionInfo() {
		barHeightMultiplier = height;
		int barSpace = 0;
		angleRot.set(MathUtils.cosDeg(rotation), MathUtils.sinDeg(rotation));
		
		barWidth = MathUtils.round((float) width/(float) barCount);
		barWidth -= spaceBetweenBars;
		for (int i = 0; i < bars.length; i++) {
			barSpace = i*(barWidth+spaceBetweenBars);
			if (flip) {
				bars[i].setRotation(rotation+180);
			} else {
				bars[i].setRotation(rotation);
			}
			if (reverse) {
				bars[bars.length-i-1].setPosition(x + barSpace*angleRot.x, y + barSpace*angleRot.y);
			} else {
				bars[i].setPosition(x + barSpace*angleRot.x, y + barSpace*angleRot.y);
			}
			for (int mirrorIndex = 0; mirrorIndex < mirrors.size; mirrorIndex++) {
				mirrors.get(mirrorIndex).position(i, barWidth, spaceBetweenBars);
			}
		}

	}
	
	public float getActualWidth() {
		return (barWidth+spaceBetweenBars)*(barCount - 1);
	}
	
	public void setColor(Color color) {
		for (int i = 0; i < bars.length; i++) {
			bars[i].setColor(color);
		}
		for (int i = 0; i < mirrors.size; i++) {
			mirrors.get(i).setColor(color);
		}
	}
	
	public void setColor(float r, float g, float b, float a) {
		for (int i = 0; i < bars.length; i++) {
			bars[i].setColor(r, g, b, a);
		}
		for (int i = 0; i < mirrors.size; i++) {
			mirrors.get(i).setColor(r, g, b, a);
		}
	}
	
	public void setRotation(float rotation) {
		this.rotation = rotation;
	}
	
	public void flip() {
		flip = !flip;
	}
	
	public boolean isFlipped() {
		return flip;
	}
	
	public void addMirrorVisualizer(MirrorVisualizer mirror) {
		updatePositionInfo();
		mirror.setup(bars, x, y, rotation);
		mirrors.add(mirror);
		x = (int) (((width - getActualWidth())/2f));
	}
	
	public void removeMirrorVisualizer(MirrorVisualizer mirror) {
		mirrors.removeValue(mirror, true);
	}
	
	public void setSpaceBetweenBars(int spaceBetweenBars) {
		this.spaceBetweenBars = spaceBetweenBars;
	}
	
	public int getBarWidth() {
		return barWidth;
	}
	
	public void reverse() {
		reverse = reverse ? false : true;
	}
	
	public boolean isReversed() {
		return reverse;
	}
	
	public Sprite[] getBars() {
		return bars;
	}
	
	public float getCurrentAvg() {
		return currentAvg;
	}
	
	public float getMaxAvgHeight() {
		return maxAvgHeight;
	}
	
	public int getSpaceBetweenBars() {
		return spaceBetweenBars;
	}
	
	public void setX(float x) {
		this.x = x;
	}
	
	public float getX() {
		return x;
	}
	
	public void setY(float y) {
		this.y = y;
	}
	
	public float getY() {
		return y;
	}
	
	public void setWidth(float width) {
		this.width = width;
	}
	
	public float getWidth() {
		return width;
	}
	
	public void setHeight(float height) {
		this.height = height;
	}
	
	public float getHeight() {
		return height;
	}

	public void calcPCMData() {
		visualizer.calcPCMData();
	}

	public void calculate() {
		visualizer.fft();
	}
	
	public Visualizer getVisualizer() {
		return visualizer;
	}
}
