package zero1hd.rhythmbullet.desktop.audio;

import java.util.Comparator;
import java.util.Observable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Sort;

import zero1hd.rhythmbullet.audio.MusicManager;

public class MusicList extends Observable {
	private Array<FileHandle> musicArray;
	private String searchPath;
	private boolean searched;
	private ExecutorService exec;
	
	public MusicList() {
		musicArray = new Array<>();
		exec = Executors.newSingleThreadExecutor();
	}
	
	private Array<FileHandle> recursiveMusicFileList(FileHandle fileHandle) {
		Array<FileHandle> musicFiles = new Array<>();
		FileHandle[] files = fileHandle.list();
		for (int i = 0; i < files.length; i++) {
			if (files[i].isDirectory()) {
				musicFiles.addAll(recursiveMusicFileList(files[i]));
			} else {
				if (files[i].extension().equalsIgnoreCase("wav") || files[i].extension().equalsIgnoreCase("mp3")) {
					musicFiles.add(files[i]);
				}
			}
		}
		return musicFiles;
	}
	
	/**
	 * refreshes song list, notifies any observers of the refresh while passing no argument to the notifier.
	 * Blocking.
	 */
	public void refresh(boolean notifyOnCompletion) {
		searched = false;
		musicArray.clear();
		Gdx.app.debug("SongController", "Searching path: " + searchPath);
		if (Gdx.files.absolute(searchPath).exists() && Gdx.files.absolute(searchPath).isDirectory()) {
			musicArray.addAll(recursiveMusicFileList(Gdx.files.absolute(searchPath)));
		}
		if (!Gdx.files.external("RhythmBullet/Alan Walker - Spectre.mp3").exists()) {
			Gdx.files.internal("music/Alan Walker - Spectre.mp3").copyTo(Gdx.files.external("RhythmBullet/Alan Walker - Spectre.mp3"));
		}
		musicArray.add(Gdx.files.external("RhythmBullet/Alan Walker - Spectre.mp3"));
		setChanged();
		Sort.instance().sort(musicArray, new Comparator<FileHandle>() {
			@Override
			public int compare(FileHandle o1, FileHandle o2) {
				return o1.nameWithoutExtension().compareTo(o2.nameWithoutExtension());
			}
		});
		
		searched = true;
		
		if (notifyOnCompletion) {
			notifyObservers();
		}
	}
	
	/**
	 * Wrapper method that uses asynch refresh.
	 * Also notifies listeners that are on the main thread.
	 */
	public void asynchRefresh() {
		searched = false;
		exec.submit(() -> {
			refresh(false);
			Gdx.app.debug("Asynch-MusicList", "Async refresh done. Notification has been sent.");
			Gdx.app.postRunnable(() -> {
				notifyObservers();
				searched = true;
			});
		});
	}
	
	public void setSearchPath(String searchPath) {
		this.searchPath = searchPath;
		setChanged();
	}
	
	/**
	 * creates an audio manager for the given music file.
	 * @param file
	 * @return a music manager of the given music file.
	 */
	public MusicManager getAudioData(FileHandle file) {
		if (file.extension().equalsIgnoreCase("wav")) {
			return new WAVManager(file);
		} else if (file.extension().equalsIgnoreCase("mp3")) {
			return new MP3AudioProcessor(file);
		}
		return null;
	}
	
	public MusicManager getMusicManagerFromIndex(int index) {
		if (!searched) Gdx.app.debug("SongList", "Warning, this list hasn't even searched yet...");
		return getAudioData(musicArray.get(index));
	}
	
	public int getTotal() {
		return musicArray.size;
	}
	
	public FileHandle getSongFileHandleFromIndex(int index) {
		if (!searched) Gdx.app.debug("SongList", "Warning, this list hasn't even searched yet...");
		return musicArray.get(index);
	}
	
	public Array<FileHandle> getMusicArray() {
		return musicArray;
	}
	
	public boolean isSearched() {
		return searched;
	}
	
}
